<?php

namespace Teknisa\TCO\Services;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;

class Tco00800ProspectService extends \Teknisa\GER\Controllers\TeknisaCrud {
    protected $connection;
    protected $environment;
    protected $crudService;
    protected $entityManager;
    protected $dataSourceManager;
    protected $dataSourceName;

    public function __construct($crudService, ParameterBag $parameterBag, $environment, $entityManager, $dataSourceManager) {
        parent::__construct($crudService, $parameterBag);
        $this->crudService       = $crudService;
        $this->connection        = $entityManager->getConnection();
        $this->environment       = $environment;
        $this->entityManager     = $entityManager;
        $this->dataSourceManager = $dataSourceManager;
    }

    /**
     * Salva o registro de prospect da forma correta.
     * 
     * @param {DTO\Request} $request Requisição vinda 
     *  do cliente.
     * @param {DTO\Response} $response Resposta à requisição
     *  vinda do cliente.
     */
    public function saveProspect(DTO\Request $request, DTO\Response $response){
        $this->dataSourceName = "prospect";
        parent::save($request, $response);
        
        $register = $request->getDataSet()->getRows()[0];
        $isNew    = $register->get('__is_new');

        $hasFilledField = $this->hasFilledAddressField($register);
        
        if($isNew && $hasFilledField){
            $returnedCdProspect = $response->getDataSets()[0]->getRows()[0]['CDPROSPECT'];
            $this->saveAddress($request, $response, $returnedCdProspect, 'P');
        }
    }

    /**
     * Verifica o preenchimento de pelo menos
     * um dos campos de endereço no registro 
     * passado por parâmetro.
     * 
     * @param {DTO\Row} $register Registro contendo
     *  dados.
     * 
     * @return {Boolean} 'true' caso esteja preenchido
     *  algum dos fields de endereço.
     */
    public function hasFilledAddressField($register){
        $addressFields = array(
            "NRCEPPROS",
            "DSENDEPROS",
            "NRIMOVELENDERECO",
            "DSCOMPENDPRO",
            "CDPAIS",
            "SGESTADO",
            "CDMUNICIPIO",
            "CDBAIRRO",
            "DSEMAILPROS",
            "NRTELEPROS"
        );

        $hasFilledField = false;

        foreach($addressFields as $addField){
            if($register->has($addField) && ($register->get($addField)!=''))
                $hasFilledField = true;
        }

        return $hasFilledField;
    }

    /**
     * Verifica se um prospect existe utilizando como 
     * parâmetro o CDPROSPECT ou o NRINSJURPROS, de acordo
     * com o que é passado para '$searchParam';
     * 
     * @param {String} $searchParam Parâmetro de busca, 'C'
     *  para prospect e outro valor para NRINSJURPROS.
     * @param {String} $paramValue Valor do parâmetro a ser
     *  utilizado para busca.
     * 
     * @return {Array|Boolean} Array de prospects caso existam
     *  ou false caso não exista.
     */
    public function existsProspect($searchParam, $paramValue){
        $searchParam = $searchParam == 'C' ? 'CDPROSPECT' : "NRINSJURPROS";

        $conditions = array(
            array(
                "columnName" => $searchParam, 
                "operator"   => "=",
                "value"      => $paramValue
            )
        );

        $filterCriteria = new FilterCriteria("prospect", $conditions);
        $dataSet = $this->dataSourceManager->findBy($filterCriteria);

        return sizeof($dataSet->getRows()) ? $dataSet->getRows() : false;
    }


    /**
     * Verifica se um tipo de endereço já foi cadastrado
     * para o prospect em questão.
     * 
     * @param {String} $cdProspect Código do prospect
     *  a ser verificado.
     * @param {String} $idTipoEnde Tipo de endereço 
     *  a ser verificado.
     * 
     * @return {Boolean} Estado de existência do tipo 
     *  de endereço.
     */
    public function existsEndeProspect($cdProspect, $idTipoEnde){

        $conditions = array(
            array(
                "columnName" => 'CDPROSPECT', 
                "operator"   => "=",
                "value"      => $cdProspect
            ),
            array(
                "columnName" => 'IDTPENDEPROS', 
                "operator"   => "=",
                "value"      => $idTipoEnde
            )
        );
        
        $filterCriteria = new FilterCriteria("endeprospect", $conditions);
        $dataSet = $this->dataSourceManager->findBy($filterCriteria);
        
        return isset($dataSet->getRows()[0]);
    }
    
    /**
     * Função responsável por remover prospects.
     * 
     * @param {Request} $request Request do usuário.
     * @param {Response} $response Response da requisição.
     */
    public function removeProspect(DTO\Request $request, DTO\Response $response){
        $this->dataSourceName = 'prospect';
        parent::delete($request, $response);
    }

    /**
     * Função responsável por retornar se prospect possui
     * registros filhos em outras tabelas
     * 
     * @param {String} $cdProspect Código do prospect
     *  a ser pesquisado.
     * 
     * @return {Array{String}} Registros onde o 
     *  prospect é utilizado.
     */
    public function getProspectDependencies($cdProspect){
        $sql = "SELECT 'TCO00800_PROSPECT_USED_PROPCOM' AS MESSAGECODE,
                    CDPROPOSTACOMERCIAL AS PARAM1
                FROM PROPOSTACOMERCIAL
                WHERE CDPROSPECT =:CDPROSPECT";

        $params = array(
            "CDPROSPECT" => $cdProspect
        );

        $query = $this->entityManager->getConnection()->prepare($sql);
        $query->execute($params);
        $result = $query->fetchAll();

        return $result;
    }

    /**
     * Salva alteração/inserção do endereço do prospect,
     * fabricando uma nova requisição para salvar eliminando
     * parâmetros de novoCodigo.
     * 
     * @param {DTO\Request} $request Requisição vinda do 
     *  cliente.
     * @param {DTO\Response} $response Resposta à requisição
     *  do cliente.
     * @param {String} $cdProspect Código do referido 
     *  prospect.
     * @param {String} $idTipoEnde Identificador do tipo
     *  de endereço.
     */
    public function saveAddress($request, $response, $cdProspect, $idTipoEnde){
        if(!($request instanceof DTO\Request\Row)){
            $reqDtSet   = $request->getDataSet()->getRows();
            $reqMethod  = $request->getMethod();
            $reqUserId  = $request->getUserId();
            $reqRPath   = $request->getRoutePath();

            $reqDtSet[0]->remove('__new_code_pk');
            $reqDtSet = new DataSet('endeprospect', $reqDtSet);
            $request  = new DTO\Request\DataSet($reqDtSet, $reqMethod, $reqRPath, $reqUserId);
        }
        
        $rowToSave = $this->crudService->getRowsToSave($request, 'endeprospect');
        
        $rowToSave[0]['CDPROSPECT']   = $cdProspect;
        $rowToSave[0]['IDTPENDEPROS'] = $idTipoEnde;
        
        $dataSet = new DataSet('endeprospect', $rowToSave);
        $this->dataSourceManager->persist($dataSet);
    }


    /**
     * Função responsável por remover endereços de
     * prospects.
     * 
     * @param {Request} $request Request do usuário.
     * @param {Response} $response Response da requisição.
     */
    public function removeAddress(DTO\Request $request, DTO\Response $response){
        $this->dataSourceName = 'endeprospect';
        parent::delete($request, $response);
    }
}